#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <usb.h>
#include "usb_my.h"

static int
usbGetStringAscii (usb_dev_handle * dev, int index, int langid, char *buf,
		   int buflen)
{
  char buffer[256];
  int rval, i;

  if ((rval =
       usb_control_msg (dev, USB_ENDPOINT_IN, USB_REQ_GET_DESCRIPTOR,
			(USB_DT_STRING << 8) + index, langid, buffer,
			sizeof (buffer), 1000)) < 0)
    return rval;
  if (buffer[1] != USB_DT_STRING)
    return 0;
  if ((unsigned char) buffer[0] < rval)
    rval = (unsigned char) buffer[0];
  rval /= 2;
  /* lossy conversion to ISO Latin1 */
  for (i = 1; i < rval; i++)
    {
      if (i > buflen)		/* destination buffer overflow */
	break;
      buf[i - 1] = buffer[2 * i];
      if (buffer[2 * i + 1] != 0)	/* outside of ISO Latin1 range */
	buf[i - 1] = '?';
    }
  buf[i - 1] = 0;
  return i - 1;
}


usb_dev_handle *
findDevice (void)
{
  struct usb_bus *bus;
  struct usb_device *dev;
  usb_dev_handle *handle = 0;

  usb_find_busses ();
  usb_find_devices ();
  for (bus = usb_busses; bus; bus = bus->next)
    {
      for (dev = bus->devices; dev; dev = dev->next)
	{
	  if (dev->descriptor.idVendor == USBDEV_SHARED_VENDOR
	      && dev->descriptor.idProduct == USBDEV_SHARED_PRODUCT)
	    {
	      char string[256];
	      int len;
	      handle = usb_open (dev);
	      if (!handle)
		{
		  fprintf (stderr, "Warning: cannot open USB device: %s\n",
			   usb_strerror ());
		  continue;
		}
	      len =
		usbGetStringAscii (handle, dev->descriptor.iManufacturer,
				   0x0409, string, sizeof (string));
	      if (len < 0)
		{
		  fprintf (stderr,
			   "warning: cannot query manufacturer for device: %s\n",
			   usb_strerror ());
		  goto skipDevice;
		}
	      fprintf (stderr, "seen device from vendor ->%s<-\n", string);
	      if (strcmp (string, "Fhunter") != 0)
		goto skipDevice;
	      len =
		usbGetStringAscii (handle, dev->descriptor.iProduct, 0x0409,
				   string, sizeof (string));
	      if (len < 0)
		{
		  fprintf (stderr,
			   "warning: cannot query product for device: %s\n",
			   usb_strerror ());
		  goto skipDevice;
		}
	      fprintf (stderr, "seen product ->%s<-\n", string);
	      if (strcmp (string, "HID LCD") == 0)
		break;
	    skipDevice:
	      usb_close (handle);
	      handle = NULL;
	    }
	}
      if (handle)
	break;
    }
  if (!handle)
    fprintf (stderr, "Could not find USB device Fhunter/HID LCD\n");
  return handle;
}
static int
usb_transmit (usb_dev_handle * usbhandle, unsigned char receive,
	      unsigned char functionid, unsigned char send[4],
	      unsigned char *buffer, int buffersize)
{
  int nbytes;
  nbytes = usb_control_msg (usbhandle,
			    USB_TYPE_VENDOR | USB_RECIP_DEVICE | (receive <<
								  7),
			    functionid, (send[1] << 8) | send[0],
			    (send[3] << 8) | send[2], buffer, buffersize,
			    5000);
  if (nbytes < 0)
    fprintf (stderr, "error: transmit: %s\n", usb_strerror ());
  return nbytes;

};

void
send_data_to_device (usb_dev_handle * handle, unsigned char *buffer,
		     unsigned char length)
{
/*-----------------*/
  int n;
  unsigned char cmd[4];
  int wbytes = length+2;
  int blocksize;
  int function;

  function = PSCMD_LCD;
  while (wbytes) {
    blocksize = wbytes;
    wbytes = 0;
    cmd[0] = 0;
    cmd[1] = 0;
    cmd[2] = 0;
    cmd[3] = 0;
    n = usb_transmit(handle,0, PSCMD_LCD, cmd, buffer, blocksize);
    if (n != blocksize) {
      fprintf(stderr, "error: wrong count at writing %x\n",n);
      exit(1);
    }
  }
  return;
};
