/* Name: main.c
 * Project: HID-Test
 * Author: Christian Starkjohann
 * Creation Date: 2006-02-02
 * Tabsize: 4
 * Copyright: (c) 2006 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: Proprietary, free under certain conditions. See Documentation.
 * This Revision: $Id: main.c 112 2006-02-07 21:30:04Z cs $
 */

#define F_CPU   12000000L	/* evaluation board runs on 4MHz */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>

#include "usbdrv.h"
#include "oddebug.h"

#define BV(bit)		(1<<(bit))
#define cbi(reg,bit)	reg &= ~(BV(bit))
#define sbi(reg,bit)	reg |= (BV(bit))

#define S_RECEIVE_LCD_DATA 	1
#define S_NORMAL 		0

/*Prototypes*/
uchar state = S_NORMAL;
static unsigned int toreceive = 0;
uchar buffer[100];
uchar buffer_ptr = 0;
uchar controller = 0;
uchar ledstatus = 0;		/* 0- off 1-on 2-blink */
static uchar replyBuf[8];

static void writedata (unsigned char cmd, char chip);
static void writecommand (unsigned char cmd, char chip);
/* ----------------------- hardware I/O abstraction ------------------------ */

/* pin assignments:
PB0	LCD Enable
PB1	LCD ChipSelect 0
PB2	LCD ChipSelect 1
PB3	LCD Adress0		MOSI
PB4	LCD Data 6		MISO
PB5 	LCD Data 7		SCK

PC0	LCD Data 0
PC1	LCD Data 1
PC2	LCD Data 2
PC3	LCD Data 3
PC4	LCD Data 4
PC5	LCD Data 5
PC6	LCD Reset		Reset

PD0	USB-
PD1	LED/Debug Tx
PD2	USB+ (int0)
PD3	Input1 
PD4	Input2 
PD5	Input3 
PD6	Input4 
PD7	Input5 
*/

void
delay_ms (unsigned short ms)	/*   */
{
  uint8_t inner, inner1;

  while (ms)
    {
      inner = 50;
      while (inner)
	{
	  inner--;
	  inner1 = 50;
	  while (inner1)
	    {
	      inner1--;
	    }
	}
      ms--;
    }
}

static void
toggleled (void)
{
  PORTD ^= 2;
};

static void
setled (uchar status)
{
  if (status)
    {
      PORTD &= ~2;
    }
  else
    {
      PORTD |= 2;
    };
};


static void
hardwareInit (void)
{
  uchar i, j;
  DDRB = 0xff;
  DDRC = 0xff;
  PORTC = 0;
  PORTB = 0;
  DDRD = 0x07;			/* 0000 0111 bin: all pins input except USB (-> USB reset) */
  PORTD = 0xfa;			/* 1111 1010 bin: activate pull-ups except on USB lines */

  j = 0;
  while (--j)
    {				/* USB Reset by device only required on Watchdog Reset */
      i = 0;
      while (--i);		/* delay >10ms for USB reset */
    }
  DDRD = 0x02;			/* 0000 0010 bin: remove USB reset condition */
  /* configure timer 0 for a rate of 12M/(1024 * 256) = 45.78 Hz (~22ms) */
  TCCR0 = 5;			/* prescaler = 1024 */
}

static void
screen_gotoxy (uchar x, uchar y)
{
  uchar controller;
  controller = (x >= 63) ? 1 : 0;
  x = x & 0x3f;
  y = y & 0x07;
  writecommand (0xb8 + y, controller);
  writecommand (0x40 + x, controller);
};

static void
write_screen_data (uchar * data, uchar len)
{
  uchar i, start_x;
  uchar controller;
  start_x = data[0];
  if (start_x > 127)
    return;
  if (data[1] > 63)
    return;
  screen_gotoxy (data[0], data[1]);
  for (i = 2; i < len; i++)
    {
      if ((data[0] + i - 2) > 127)
	return;
      if ((i > 2) && ((start_x + i - 3) <= 63) && ((start_x + i - 2) > 63))
	{

	  screen_gotoxy (data[0] + i - 2, data[1]);
	}
      controller = ((start_x + i - 2) > 63) ? 1 : 0;
      writedata (data[i], controller);
    };

  return;
};




static void
timerinterrupt (void)
{
  switch (ledstatus)
    {
    case 0:
      setled (0);
      break;
    case 1:
      setled (1);
      break;
    case 2:
      toggleled ();
      break;
    };
};


/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

uchar
usbFunctionSetup (uchar data[8])
{
  uchar len=0;
  switch (data[1])
    {
    case 0:
      {
	replyBuf[0] = data[2];
	replyBuf[1] = data[3];
	len=2;
	break;
      };
    case 1:
      {
	replyBuf[0] = PIND;
	replyBuf[1] = 'K';
	len=2;
	break;
      };
    case 2:
      {
	ledstatus = data[2];
	replyBuf[0] = 'L';
	replyBuf[1] = 'E';
	len=2;
	break;
      };
    case 3:
      {
        if(state==S_NORMAL){
		buffer_ptr=0;
		toreceive = (data[7]<<8)|data[6];
		toreceive;
	};
	state = S_RECEIVE_LCD_DATA;
	len=0xff;
	break;
      };
    case 6:
      {
	writedata (data[2], controller);
	break;
      };
    case 7:
      {
	writecommand (data[2], controller);
	break;
      };
    case 8:
      {
	controller = data[2];
	break;
      };
    }
  usbMsgPtr = replyBuf;
  return len;
}

uchar
usbFunctionWrite (uchar * data, uchar len)
{

  uchar i;
  /* check if device is in correct write state */
  if (state != S_RECEIVE_LCD_DATA)
    {
      return 0xff;
    }

  for (i = 0; i < len; i++)
    {
      buffer[buffer_ptr] = data[i];
      toreceive--;
      if (toreceive == 0)
	{
	  toggleled();
	  write_screen_data (buffer, buffer_ptr);
	  buffer_ptr = 0;
	  state= S_NORMAL;
	  return 0;
	}
	buffer_ptr++;
    };
  return 0;
}


static void
writecommand (unsigned char cmd, char chip)  /**/
{
  cbi (PORTB, PB0);		//E
  cbi (PORTB, PB1);		//CS0
  cbi (PORTB, PB2);		//CS1
  cbi (PORTB, PB3);		//A0

  if (chip)
    {
      sbi (PORTB, PB2);
    }
  else
    {
      sbi (PORTB, PB1);
    };
  PORTC = cmd & 0x3F;
  if (cmd & 0x80)
    {
      sbi (PORTB, PB5);
    }
  else
    {
      cbi (PORTB, PB5);
    };
  if (cmd & 0x40)
    {
      sbi (PORTB, PB4);
    }
  else
    {
      cbi (PORTB, PB4);
    };
  delay_ms (1);
  sbi (PORTB, PB0);
/*  delay_ms (1);*/
  cbi (PORTD, PB0);
  cbi (PORTB, PB1);
  cbi (PORTB, PB2);
};

static void
writedata (unsigned char cmd, char chip)   /**/
{
  cbi (PORTB, PB0);		//E
  cbi (PORTB, PB1);		//CS0
  cbi (PORTB, PB2);		//CS1
  sbi (PORTB, PB3);		//A0

  if (chip)
    {
      sbi (PORTB, PB2);
    }
  else
    {
      sbi (PORTB, PB1);
    };
  PORTC = cmd & 0x3F;
  if (cmd & 0x80)
    {
      sbi (PORTB, PB5);
    }
  else
    {
      cbi (PORTB, PB5);
    };
  if (cmd & 0x40)
    {
      sbi (PORTB, PB4);
    }
  else
    {
      cbi (PORTB, PB4);
    };
  delay_ms (1);
  sbi (PORTB, PB0);
/*  delay_ms (1);*/
  cbi (PORTD, PB0);
  cbi (PORTB, PB1);
  cbi (PORTB, PB2);
};

/* ------------------------------------------------------------------------- */

int
main (void)
{
  uchar count = 0;
  wdt_enable (WDTO_2S);
  hardwareInit ();
  writecommand (0x3f, 0);
  writecommand (0x3f, 1);
  delay_ms (1);
  writecommand (0xc0, 0);
  writecommand (0xc0, 1);
  delay_ms (1);
  writecommand (0xb8, 0);
  writecommand (0xb8, 1);
  delay_ms (1);
  writecommand (0x40, 0);
  writecommand (0x40, 1);
  usbInit ();
  sei ();
  for (;;)
    {				/* main event loop */
      wdt_reset ();
      usbPoll ();
      if (TIFR & (1 << TOV0))
	{			/* 22 ms timer */
	  TIFR = 1 << TOV0;
	  count++;
	  if (count > 25)
	    {
	      timerinterrupt ();
	      count = 0;
	    };
	}
    }
  return 0;
}

/* ------------------------------------------------------------------------- */
